@@ -46,6 +46,12 @@ eel_uri_is_trash (const char *uri)
 	return g_str_has_prefix (uri, "trash:");
 }
 
+gboolean
+eel_uri_is_recent (const char *uri)
+{
+	return g_str_has_prefix (uri, "recent:");
+}
+
 gboolean
 eel_uri_is_search (const char *uri)
 {
