@@ -115,6 +115,32 @@ nautilus_get_user_directory (void)
 	return user_directory;
 }
 
+/**
+ * nautilus_get_accel_map_file:
+ * 
+ * Get the path for the filename containing nautilus accelerator map.
+ * The filename need not exist.
+ *
+ * Return value: the filename path
+ **/
+char *
+nautilus_get_accel_map_file (void)
+{
+	return g_build_filename (g_get_user_config_dir (), "nautilus", "accels", NULL);
+}
+
+/**
+ * nautilus_get_scripts_directory_path:
+ *
+ * Get the path for the directory containing nautilus scripts.
+ *
+ * Return value: the directory path containing nautilus scripts
+ **/
+char *
+nautilus_get_scripts_directory_path (void)
+{
+	return g_build_filename (g_get_user_data_dir (), "nautilus", "scripts", NULL);
+}
 
 typedef struct {
 	char *type;
