@@ -28,6 +28,7 @@
 #include "nautilus-file.h"
 #include "nautilus-file-private.h"
 #include "nautilus-file-utilities.h"
+#include "nautilus-search-provider.h"
 #include "nautilus-search-engine.h"
 #include <eel/eel-glib-extensions.h>
 #include <gtk/gtk.h>
@@ -139,17 +140,17 @@ start_or_stop_search_engine (NautilusSearchDirectory *search, gboolean adding)
 		search->details->search_running = TRUE;
 		search->details->search_finished = FALSE;
 		ensure_search_engine (search);
-		nautilus_search_engine_set_query (search->details->engine, search->details->query);
+		nautilus_search_provider_set_query (NAUTILUS_SEARCH_PROVIDER (search->details->engine), search->details->query);
 
 		reset_file_list (search);
 
-		nautilus_search_engine_start (search->details->engine);
+		nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (search->details->engine));
 	} else if (!adding && (!search->details->monitor_list ||
 		   !search->details->pending_callback_list) &&
 		   search->details->engine &&
 		   search->details->search_running) {
 		search->details->search_running = FALSE;
-		nautilus_search_engine_stop (search->details->engine);
+		nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (search->details->engine));
 
 		reset_file_list (search);
 	}
@@ -467,22 +468,26 @@ search_engine_hits_added (NautilusSearchEngine *engine, GList *hits,
 	GList *hit_list;
 	GList *file_list;
 	NautilusFile *file;
-	char *uri;
 	SearchMonitor *monitor;
 	GList *monitor_list;
 
 	file_list = NULL;
 
 	for (hit_list = hits; hit_list != NULL; hit_list = hit_list->next) {
-		uri = hit_list->data;
+		NautilusSearchHit *hit = hit_list->data;
+		const char *uri;
 
+		uri = nautilus_search_hit_get_uri (hit);
 		if (g_str_has_suffix (uri, NAUTILUS_SAVED_SEARCH_EXTENSION)) {
 			/* Never return saved searches themselves as hits */
 			continue;
 		}
-		
+
+		nautilus_search_hit_compute_scores (hit, search->details->query);
+
 		file = nautilus_file_get_by_uri (uri);
-		
+		nautilus_file_set_search_relevance (file, nautilus_search_hit_get_relevance (hit));
+
 		for (monitor_list = search->details->monitor_list; monitor_list; monitor_list = monitor_list->next) {
 			monitor = monitor_list->data;
 
@@ -512,13 +517,15 @@ search_engine_hits_subtracted (NautilusSearchEngine *engine, GList *hits,
 	GList *monitor_list;
 	SearchMonitor *monitor;
 	GList *file_list;
-	char *uri;
 	NautilusFile *file;
 
 	file_list = NULL;
 
 	for (hit_list = hits; hit_list != NULL; hit_list = hit_list->next) {
-		uri = hit_list->data;
+		NautilusSearchHit *hit = hit_list->data;
+		const char *uri;
+
+		uri = nautilus_search_hit_get_uri (hit);
 		file = nautilus_file_get_by_uri (uri);
 
 		for (monitor_list = search->details->monitor_list; monitor_list; 
@@ -603,7 +610,7 @@ search_force_reload (NautilusDirectory *directory)
 	reset_file_list (search);
 	
 	if (search->details->search_running) {
-		nautilus_search_engine_stop (search->details->engine);
+		nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (search->details->engine));
 		search->details->search_running = FALSE;
 	}
 }
@@ -690,7 +697,7 @@ search_dispose (GObject *object)
 
 	if (search->details->engine) {
 		if (search->details->search_running) {
-			nautilus_search_engine_stop (search->details->engine);
+			nautilus_search_provider_stop (NAUTILUS_SEARCH_PROVIDER (search->details->engine));
 		}
 		
 		g_object_unref (search->details->engine);
