@@ -22,6 +22,8 @@
  */
 
 #include <config.h>
+#include "nautilus-search-hit.h"
+#include "nautilus-search-provider.h"
 #include "nautilus-search-engine-tracker.h"
 #include <string.h>
 #include <gio/gio.h>
@@ -36,9 +38,13 @@ struct NautilusSearchEngineTrackerDetails {
 	GCancellable  *cancellable;
 };
 
-G_DEFINE_TYPE (NautilusSearchEngineTracker,
-	       nautilus_search_engine_tracker,
-	       NAUTILUS_TYPE_SEARCH_ENGINE);
+static void nautilus_search_provider_init (NautilusSearchProviderIface  *iface);
+
+G_DEFINE_TYPE_WITH_CODE (NautilusSearchEngineTracker,
+			 nautilus_search_engine_tracker,
+			 G_TYPE_OBJECT,
+			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SEARCH_PROVIDER,
+						nautilus_search_provider_init))
 
 static void
 finalize (GObject *object)
@@ -80,6 +86,12 @@ cursor_callback (GObject      *object,
 	NautilusSearchEngineTracker *tracker;
 	GError *error = NULL;
 	TrackerSparqlCursor *cursor;
+	NautilusSearchHit *hit;
+	const char *uri;
+	const char *mtime_str;
+	const char *atime_str;
+	GTimeVal tv;
+	gdouble rank;
 	GList *hits;
 	gboolean success;
 
@@ -89,28 +101,48 @@ cursor_callback (GObject      *object,
 	success = tracker_sparql_cursor_next_finish (cursor, result, &error);
 
 	if (error && !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		tracker->details->query_pending = FALSE;
-		nautilus_search_engine_error (NAUTILUS_SEARCH_ENGINE (tracker), error->message);
-		g_error_free (error);
-		g_object_unref (cursor);
-
-		return;
+		nautilus_search_provider_error (NAUTILUS_SEARCH_PROVIDER (tracker), error->message);
 	}
 
 	g_clear_error (&error);
 
 	if (!success) {
 		tracker->details->query_pending = FALSE;
-		nautilus_search_engine_finished (NAUTILUS_SEARCH_ENGINE (tracker));
+		nautilus_search_provider_finished (NAUTILUS_SEARCH_PROVIDER (tracker));
 		g_object_unref (cursor);
 
 		return;
 	}
 
 	/* We iterate result by result, not n at a time. */
-	hits = g_list_append (NULL, (gchar*) tracker_sparql_cursor_get_string (cursor, 0, NULL));
-	nautilus_search_engine_hits_added (NAUTILUS_SEARCH_ENGINE (tracker), hits);
+	uri = tracker_sparql_cursor_get_string (cursor, 0, NULL);
+	rank = tracker_sparql_cursor_get_double (cursor, 1);
+	mtime_str = tracker_sparql_cursor_get_string (cursor, 2, NULL);
+	atime_str = tracker_sparql_cursor_get_string (cursor, 3, NULL);
+
+	hit = nautilus_search_hit_new (uri);
+	nautilus_search_hit_set_fts_rank (hit, rank);
+	if (g_time_val_from_iso8601 (mtime_str, &tv)) {
+		GDateTime *dt;
+		dt = g_date_time_new_from_timeval_local (&tv);
+		nautilus_search_hit_set_modification_time (hit, dt);
+		g_date_time_unref (dt);
+	} else {
+		g_warning ("unable to parse mtime: %s", mtime_str);
+	}
+	if (g_time_val_from_iso8601 (atime_str, &tv)) {
+		GDateTime *dt;
+		dt = g_date_time_new_from_timeval_local (&tv);
+		nautilus_search_hit_set_access_time (hit, dt);
+		g_date_time_unref (dt);
+	} else {
+		g_warning ("unable to parse atime: %s", atime_str);
+	}
+
+	hits = g_list_append (NULL, hit);
+	nautilus_search_provider_hits_added (NAUTILUS_SEARCH_PROVIDER (tracker), hits);
 	g_list_free (hits);
+	g_object_unref (hit);
 
 	/* Get next */
 	cursor_next (tracker, cursor);
@@ -134,17 +166,14 @@ query_callback (GObject      *object,
 	                                                 &error);
 
 	if (error && !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		tracker->details->query_pending = FALSE;
-		nautilus_search_engine_error (NAUTILUS_SEARCH_ENGINE (tracker), error->message);
-		g_error_free (error);
-		return;
+		nautilus_search_provider_error (NAUTILUS_SEARCH_PROVIDER (tracker), error->message);
 	}
 
 	g_clear_error (&error);
 
 	if (!cursor) {
 		tracker->details->query_pending = FALSE;
-		nautilus_search_engine_finished (NAUTILUS_SEARCH_ENGINE (tracker));
+		nautilus_search_provider_finished (NAUTILUS_SEARCH_PROVIDER (tracker));
 		return;
 	}
 
@@ -152,7 +181,7 @@ query_callback (GObject      *object,
 }
 
 static void
-nautilus_search_engine_tracker_start (NautilusSearchEngine *engine)
+nautilus_search_engine_tracker_start (NautilusSearchProvider *provider)
 {
 	NautilusSearchEngineTracker *tracker;
 	gchar	*query_text, *search_text, *location_uri, *downcase;
@@ -160,7 +189,7 @@ nautilus_search_engine_tracker_start (NautilusSearchEngine *engine)
 	GList *mimetypes, *l;
 	gint mime_count;
 
-	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (engine);
+	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (provider);
 
 	if (tracker->details->query_pending) {
 		return;
@@ -181,7 +210,7 @@ nautilus_search_engine_tracker_start (NautilusSearchEngine *engine)
 
 	mime_count = g_list_length (mimetypes);
 
-	sparql = g_string_new ("SELECT DISTINCT nie:url(?urn) "
+	sparql = g_string_new ("SELECT DISTINCT nie:url(?urn) fts:rank(?urn) tracker:coalesce(nfo:fileLastModified(?urn), nie:contentLastModified(?urn)) AS ?mtime tracker:coalesce(nfo:fileLastAccessed(?urn), nie:contentAccessed(?urn)) AS ?atime "
 			       "WHERE {"
 			       "  ?urn a nfo:FileDataObject ;"
 			       "  tracker:available true ; ");
@@ -215,7 +244,7 @@ nautilus_search_engine_tracker_start (NautilusSearchEngine *engine)
 		g_string_append (sparql, ")");
 	}
 
-	g_string_append (sparql, ")}");
+	g_string_append (sparql, ")} ORDER BY DESC (fts:rank(?urn))");
 
 	tracker->details->cancellable = g_cancellable_new ();
 	tracker->details->query_pending = TRUE;
@@ -232,11 +261,11 @@ nautilus_search_engine_tracker_start (NautilusSearchEngine *engine)
 }
 
 static void
-nautilus_search_engine_tracker_stop (NautilusSearchEngine *engine)
+nautilus_search_engine_tracker_stop (NautilusSearchProvider *provider)
 {
 	NautilusSearchEngineTracker *tracker;
 
-	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (engine);
+	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (provider);
 	
 	if (tracker->details->query && tracker->details->query_pending) {
 		g_cancellable_cancel (tracker->details->cancellable);
@@ -246,11 +275,12 @@ nautilus_search_engine_tracker_stop (NautilusSearchEngine *engine)
 }
 
 static void
-nautilus_search_engine_tracker_set_query (NautilusSearchEngine *engine, NautilusQuery *query)
+nautilus_search_engine_tracker_set_query (NautilusSearchProvider *provider,
+					  NautilusQuery *query)
 {
 	NautilusSearchEngineTracker *tracker;
 
-	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (engine);
+	tracker = NAUTILUS_SEARCH_ENGINE_TRACKER (provider);
 
 	if (query) {
 		g_object_ref (query);
@@ -263,20 +293,22 @@ nautilus_search_engine_tracker_set_query (NautilusSearchEngine *engine, Nautilus
 	tracker->details->query = query;
 }
 
+static void
+nautilus_search_provider_init (NautilusSearchProviderIface *iface)
+{
+	iface->set_query = nautilus_search_engine_tracker_set_query;
+	iface->start = nautilus_search_engine_tracker_start;
+	iface->stop = nautilus_search_engine_tracker_stop;
+}
+
 static void
 nautilus_search_engine_tracker_class_init (NautilusSearchEngineTrackerClass *class)
 {
 	GObjectClass *gobject_class;
-	NautilusSearchEngineClass *engine_class;
 
 	gobject_class = G_OBJECT_CLASS (class);
 	gobject_class->finalize = finalize;
 
-	engine_class = NAUTILUS_SEARCH_ENGINE_CLASS (class);
-	engine_class->set_query = nautilus_search_engine_tracker_set_query;
-	engine_class->start = nautilus_search_engine_tracker_start;
-	engine_class->stop = nautilus_search_engine_tracker_stop;
-
 	g_type_class_add_private (class, sizeof (NautilusSearchEngineTrackerDetails));
 }
 
@@ -288,7 +320,7 @@ nautilus_search_engine_tracker_init (NautilusSearchEngineTracker *engine)
 }
 
 
-NautilusSearchEngine *
+NautilusSearchEngineTracker *
 nautilus_search_engine_tracker_new (void)
 {
 	NautilusSearchEngineTracker *engine;
@@ -306,5 +338,5 @@ nautilus_search_engine_tracker_new (void)
 	engine = g_object_new (NAUTILUS_TYPE_SEARCH_ENGINE_TRACKER, NULL);
 	engine->details->connection = connection;
 
-	return NAUTILUS_SEARCH_ENGINE (engine);
+	return engine;
 }
