@@ -77,6 +77,8 @@
 #define NAUTILUS_ACTION_PASTE "Paste"
 #define NAUTILUS_ACTION_PASTE_FILES_INTO "Paste Files Into"
 #define NAUTILUS_ACTION_LOCATION_PASTE_FILES_INTO "LocationPasteFilesInto"
+#define NAUTILUS_ACTION_MOVE_TO "Move To"
+#define NAUTILUS_ACTION_COPY_TO "Copy To"
 #define NAUTILUS_ACTION_RENAME "Rename"
 #define NAUTILUS_ACTION_DUPLICATE "Duplicate"
 #define NAUTILUS_ACTION_CREATE_LINK "Create Link"
