@@ -36,11 +36,11 @@
 
 #include "nautilus-bookmarks-window.h"
 #include "nautilus-connect-server-dialog.h"
-#include "nautilus-desktop-icon-view.h"
+#include "nautilus-desktop-canvas-view.h"
 #include "nautilus-desktop-window.h"
 #include "nautilus-file-management-properties.h"
 #include "nautilus-freedesktop-dbus.h"
-#include "nautilus-icon-view.h"
+#include "nautilus-canvas-view.h"
 #include "nautilus-image-properties-page.h"
 #include "nautilus-list-view.h"
 #include "nautilus-previewer.h"
@@ -910,14 +910,16 @@ static GActionEntry app_entries[] = {
 };
 
 static void
-nautilus_application_init_app_menu (NautilusApplication *self)
+nautilus_application_init_actions (NautilusApplication *self)
 {
 	GtkBuilder *builder;
 	GError *error = NULL;
+	const gchar *debug_no_app_menu;
 
 	g_action_map_add_action_entries (G_ACTION_MAP (self),
 					 app_entries, G_N_ELEMENTS (app_entries),
 					 self);
+	gtk_application_add_accelerator (GTK_APPLICATION (self), "F10", "win.gear-menu", NULL);
 
 	builder = gtk_builder_new ();
 	gtk_builder_add_from_resource (builder, "/org/gnome/nautilus/nautilus-app-menu.ui", &error);
@@ -931,6 +933,14 @@ nautilus_application_init_app_menu (NautilusApplication *self)
 	}
 
 	g_object_unref (builder);
+
+	debug_no_app_menu = g_getenv ("NAUTILUS_DEBUG_NO_APP_MENU");
+	if (debug_no_app_menu) {
+		DEBUG ("Disabling app menu GtkSetting as requested...");
+		g_object_set (gtk_settings_get_default (),
+			      "gtk-shell-shows-app-menu", FALSE,
+			      NULL);
+	}
 }
 
 static void
@@ -1352,8 +1362,8 @@ nautilus_application_startup (GApplication *app)
 	nautilus_global_preferences_init ();
 
 	/* register views */
-	nautilus_icon_view_register ();
-	nautilus_desktop_icon_view_register ();
+	nautilus_canvas_view_register ();
+	nautilus_desktop_canvas_view_register ();
 	nautilus_list_view_register ();
 #if ENABLE_EMPTY_VIEW
 	nautilus_empty_view_register ();
@@ -1394,7 +1404,7 @@ nautilus_application_startup (GApplication *app)
 
 	do_upgrades_once (self);
 
-	nautilus_application_init_app_menu (self);
+	nautilus_application_init_actions (self);
 }
 
 static void
