@@ -42,6 +42,7 @@ typedef enum {
 typedef struct {
 	NautilusQueryEditorRowType type;
 	NautilusQueryEditor *editor;
+	GtkWidget *toolbar;
 	GtkWidget *hbox;
 	GtkWidget *combo;
 
@@ -66,7 +67,6 @@ struct NautilusQueryEditorDetails {
 	gboolean change_frozen;
 	guint typing_timeout_id;
 	gboolean is_visible;
-	GtkWidget *vbox;
 
 	GtkWidget *search_current_button;
 	GtkWidget *search_all_button;
@@ -78,6 +78,7 @@ struct NautilusQueryEditorDetails {
 };
 
 enum {
+	ACTIVATED,
 	CHANGED,
 	CANCEL,
 	LAST_SIGNAL
@@ -197,6 +198,21 @@ nautilus_query_editor_handle_event (NautilusQueryEditor *editor,
 	char *old_text;
 	const char *new_text;
 
+	/* if we're focused already, no need to handle the event manually */
+	if (gtk_widget_has_focus (editor->details->entry)) {
+		return FALSE;
+	}
+
+	/* never handle these events */
+	if (event->keyval == GDK_KEY_slash || event->keyval == GDK_KEY_Delete) {
+		return FALSE;
+	}
+
+	/* don't activate search for these events */
+	if (!gtk_widget_get_visible (GTK_WIDGET (editor)) && event->keyval == GDK_KEY_space) {
+		return FALSE;
+	}
+
 	editor->details->got_preedit = FALSE;
 	if (!gtk_widget_get_realized (editor->details->entry)) {
 		gtk_widget_realize (editor->details->entry);
@@ -241,27 +257,6 @@ nautilus_query_editor_dispose (GObject *object)
 	G_OBJECT_CLASS (nautilus_query_editor_parent_class)->dispose (object);
 }
 
-static gboolean
-nautilus_query_editor_draw (GtkWidget *widget,
-			    cairo_t *cr)
-{
-	GtkStyleContext *context;
-
-	context = gtk_widget_get_style_context (widget);
-
-	gtk_render_background (context, cr, 0, 0,
-			       gtk_widget_get_allocated_width (widget),
-			       gtk_widget_get_allocated_height (widget));
-
-	gtk_render_frame (context, cr, 0, 0,
-			  gtk_widget_get_allocated_width (widget),
-			  gtk_widget_get_allocated_height (widget));
-
-	GTK_WIDGET_CLASS (nautilus_query_editor_parent_class)->draw (widget, cr);
-
-	return FALSE;
-}
-
 static void
 nautilus_query_editor_grab_focus (GtkWidget *widget)
 {
@@ -283,7 +278,6 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
         gobject_class->dispose = nautilus_query_editor_dispose;
 
 	widget_class = GTK_WIDGET_CLASS (class);
-	widget_class->draw = nautilus_query_editor_draw;
 	widget_class->grab_focus = nautilus_query_editor_grab_focus;
 
 	signals[CHANGED] =
@@ -304,6 +298,15 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 
+	signals[ACTIVATED] =
+		g_signal_new ("activated",
+		              G_TYPE_FROM_CLASS (class),
+		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		              G_STRUCT_OFFSET (NautilusQueryEditorClass, activated),
+		              NULL, NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE, 0);
+
 	binding_set = gtk_binding_set_by_class (class);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "cancel", 0);
 
@@ -322,12 +325,7 @@ nautilus_query_editor_get_location (NautilusQueryEditor *editor)
 static void
 entry_activate_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 {
-	if (editor->details->typing_timeout_id > 0) {
-		g_source_remove (editor->details->typing_timeout_id);
-		editor->details->typing_timeout_id = 0;
-	}
-
-	nautilus_query_editor_changed_force (editor, TRUE);
+	g_signal_emit (editor, signals[ACTIVATED], 0);
 }
 
 static gboolean
@@ -871,8 +869,7 @@ remove_row_cb (GtkButton *clicked_button, NautilusQueryEditorRow *row)
 	NautilusQueryEditor *editor;
 
 	editor = row->editor;
-	gtk_container_remove (GTK_CONTAINER (editor->details->vbox),
-			      row->hbox);
+	gtk_container_remove (GTK_CONTAINER (editor), row->toolbar);
 	
 	editor->details->rows = g_list_remove (editor->details->rows, row);
 
@@ -919,24 +916,32 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 			       NautilusQueryEditorRowType type)
 {
 	GtkWidget *hbox, *button, *image, *combo;
+	GtkToolItem *item;
 	NautilusQueryEditorRow *row;
 	int i;
 
 	row = g_new0 (NautilusQueryEditorRow, 1);
 	row->editor = editor;
 	row->type = type;
-	
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	row->hbox = hbox;
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
 
+	/* create the toolbar and the box container for its contents */
+	row->toolbar = gtk_toolbar_new ();
+	gtk_box_pack_start (GTK_BOX (editor), row->toolbar, TRUE, TRUE, 0);
+
+	item = gtk_tool_item_new ();
+	gtk_tool_item_set_expand (item, TRUE);
+	gtk_toolbar_insert (GTK_TOOLBAR (row->toolbar), item, -1);
+
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	gtk_container_add (GTK_CONTAINER (item), hbox);
+	row->hbox = hbox;
+
+	/* create the criterion selector combobox */
 	combo = gtk_combo_box_text_new ();
 	row->combo = combo;
 	for (i = 0; i < NAUTILUS_QUERY_EDITOR_ROW_LAST; i++) {
 		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), gettext (row_type[i].name));
 	}
-	gtk_widget_show (combo);
 	gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), row->type);
@@ -947,21 +952,24 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 			  G_CALLBACK (row_type_combo_changed_cb), row);
 	
 	create_type_widgets (row);
-	
+
+	/* create the remove row button */
 	button = gtk_button_new ();
-	image = gtk_image_new_from_icon_name ("list-remove-symbolic",
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_RAISED);
+	gtk_widget_set_tooltip_text (button,
+				     _("Remove this criterion from the search"));
+	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+
+	image = gtk_image_new_from_icon_name ("window-close-symbolic",
 					      GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add (GTK_CONTAINER (button), image);
-	gtk_widget_show (image);
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
 
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (remove_row_cb), row);
-	gtk_widget_set_tooltip_text (button,
-				     _("Remove this criterion from the search"));
-	
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+
+	/* show everything */
+	gtk_widget_show_all (row->toolbar);
 
 	return row;
 }
@@ -979,18 +987,7 @@ nautilus_query_editor_init (NautilusQueryEditor *editor)
 	editor->details = G_TYPE_INSTANCE_GET_PRIVATE (editor, NAUTILUS_TYPE_QUERY_EDITOR,
 						       NautilusQueryEditorDetails);
 
-	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
-				     GTK_STYLE_CLASS_TOOLBAR);
-	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
-				     GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
-
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (editor), GTK_ORIENTATION_VERTICAL);
-
-	editor->details->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->details->vbox), 6);
-	gtk_box_pack_start (GTK_BOX (editor), editor->details->vbox,
-			    FALSE, FALSE, 0);
-	gtk_widget_show (editor->details->vbox);
 }
 
 static void
@@ -1007,62 +1004,86 @@ on_current_button_toggled (GtkToggleButton     *button,
 	nautilus_query_editor_changed (editor);
 }
 
-static void
-finish_first_line (NautilusQueryEditor *editor, GtkWidget *hbox, gboolean use_go)
+static gboolean
+entry_key_press_event_cb (GtkWidget           *widget,
+			  GdkEventKey         *event,
+			  NautilusQueryEditor *editor)
 {
-	GtkWidget *button, *image;
-
-	button = gtk_button_new ();
-	image = gtk_image_new_from_icon_name ("list-add-symbolic",
-					      GTK_ICON_SIZE_SMALL_TOOLBAR);
-	gtk_container_add (GTK_CONTAINER (button), image);
-	gtk_widget_show (image);
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	gtk_widget_show (button);
-
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (add_new_row_cb), editor);
-	
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-
-	gtk_widget_set_tooltip_text (button,
-				     _("Add a new criterion to this search"));
-
-	editor->details->search_current_button = gtk_radio_button_new_with_label (NULL, _("Current"));
-	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_current_button), FALSE);
-	gtk_widget_show (editor->details->search_current_button);
-	editor->details->search_all_button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (editor->details->search_current_button),
-											  _("All Files"));
-	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_all_button), FALSE);
-	gtk_widget_show (editor->details->search_all_button);
-	g_signal_connect (editor->details->search_all_button, "toggled",
-			  G_CALLBACK (on_all_button_toggled), editor);
-	g_signal_connect (editor->details->search_current_button, "toggled",
-			  G_CALLBACK (on_current_button_toggled), editor);
-	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_current_button, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_all_button, FALSE, FALSE, 0);
+	if (event->keyval == GDK_KEY_Down) {
+		nautilus_window_grab_focus (NAUTILUS_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (widget))));
+	}
+	return FALSE;
 }
 
 static void
 setup_widgets (NautilusQueryEditor *editor)
 {
-	GtkWidget *hbox;
+	GtkToolItem *item;
+	GtkWidget *toolbar, *button_box, *hbox;
+	GtkWidget *button, *image;
+
+	/* create the toolbar and the box container for its contents */
+	toolbar = gtk_toolbar_new ();
+	gtk_style_context_add_class (gtk_widget_get_style_context (toolbar),
+				     GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
+	gtk_box_pack_start (GTK_BOX (editor), toolbar, TRUE, TRUE, 0);
+
+	item = gtk_tool_item_new ();
+	gtk_tool_item_set_expand (item, TRUE);
+	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
 
-	/* Create visible part: */
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (item), hbox);
 
+	/* create the search entry */
 	editor->details->entry = gtk_search_entry_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), editor->details->entry, TRUE, TRUE, 0);
 
+	g_signal_connect (editor->details->entry, "key-press-event",
+			  G_CALLBACK (entry_key_press_event_cb), editor);
 	g_signal_connect (editor->details->entry, "activate",
 			  G_CALLBACK (entry_activate_cb), editor);
 	g_signal_connect (editor->details->entry, "changed",
 			  G_CALLBACK (entry_changed_cb), editor);
-	gtk_widget_show (editor->details->entry);
 
-	finish_first_line (editor, hbox, TRUE);
+	/* create the Current/All Files selector */
+	editor->details->search_current_button = gtk_radio_button_new_with_label (NULL, _("Current"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_current_button), FALSE);
+	editor->details->search_all_button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (editor->details->search_current_button),
+											  _("All Files"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_all_button), FALSE);
+	g_signal_connect (editor->details->search_all_button, "toggled",
+			  G_CALLBACK (on_all_button_toggled), editor);
+	g_signal_connect (editor->details->search_current_button, "toggled",
+			  G_CALLBACK (on_current_button_toggled), editor);
+
+	button_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), button_box, FALSE, FALSE, 0);
+	gtk_style_context_add_class (gtk_widget_get_style_context (button_box),
+				     GTK_STYLE_CLASS_LINKED);
+	gtk_style_context_add_class (gtk_widget_get_style_context (button_box),
+				     GTK_STYLE_CLASS_RAISED);
+
+	gtk_box_pack_start (GTK_BOX (button_box), editor->details->search_current_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (button_box), editor->details->search_all_button, FALSE, FALSE, 0);
+
+	/* finally, create the add new row button */
+	button = gtk_button_new ();
+	gtk_style_context_add_class (gtk_widget_get_style_context (button),
+				     GTK_STYLE_CLASS_RAISED);
+	gtk_widget_set_tooltip_text (button,
+				     _("Add a new criterion to this search"));
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+
+	image = gtk_image_new_from_icon_name ("list-add-symbolic",
+					      GTK_ICON_SIZE_SMALL_TOOLBAR);
+	gtk_container_add (GTK_CONTAINER (button), image);
+
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (add_new_row_cb), editor);
+
+	/* show everything */
+	gtk_widget_show_all (toolbar);
 }
 
 static void
