@@ -113,7 +113,7 @@ toolbar_create_toolbutton (NautilusToolbar *self,
 	gtk_button_set_image (GTK_BUTTON (button), image);
 
 	if (create_menu) {
-		gtk_image_set_from_icon_name (GTK_IMAGE (image), "emblem-system-symbolic",
+		gtk_image_set_from_icon_name (GTK_IMAGE (image), name,
 					      GTK_ICON_SIZE_MENU);
 	} else {
 		action = gtk_action_group_get_action (self->priv->action_group, name);
@@ -205,6 +205,8 @@ nautilus_toolbar_constructed (GObject *obj)
 	tool_button = toolbar_create_toolbutton (self, TRUE, FALSE, "emblem-system-symbolic");
 	menu = gtk_ui_manager_get_widget (self->priv->ui_manager, "/ViewMenu");
 	gtk_menu_button_set_menu (GTK_MENU_BUTTON (tool_button), menu);
+	gtk_actionable_set_action_name (GTK_ACTIONABLE (tool_button), "win.gear-menu");
+
 	gtk_container_add (GTK_CONTAINER (tool_item), tool_button);
 	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
 	gtk_widget_show_all (GTK_WIDGET (tool_item));
