@@ -27,11 +27,11 @@
 #include "nautilus-window-slot.h"
 
 #include "nautilus-actions.h"
-#include "nautilus-desktop-window.h"
 #include "nautilus-toolbar.h"
 #include "nautilus-floating-bar.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-manage-views.h"
+#include "nautilus-desktop-window.h"
 
 #include <glib/gi18n.h>
 
@@ -56,6 +56,11 @@ gboolean
 nautilus_window_slot_handle_event (NautilusWindowSlot *slot,
 				   GdkEventKey        *event)
 {
+	NautilusWindow *window;
+
+	window = nautilus_window_slot_get_window (slot);
+	if (NAUTILUS_IS_DESKTOP_WINDOW (window))
+		return FALSE;
 	return nautilus_query_editor_handle_event (slot->query_editor, event);
 }
 
@@ -91,14 +96,16 @@ sync_search_directory (NautilusWindowSlot *slot)
 	nautilus_directory_unref (directory);
 }
 
-static void
+static gboolean
 sync_search_location_cb (NautilusWindow *window,
 			 GError *error,
 			 gpointer user_data)
 {
 	NautilusWindowSlot *slot = user_data;
-
-	sync_search_directory (slot);
+	if (error == NULL) {
+		sync_search_directory (slot);
+	}
+	return (error == NULL);
 }
 
 static void
@@ -133,6 +140,15 @@ query_editor_cancel_callback (NautilusQueryEditor *editor,
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (search), FALSE);
 }
 
+static void
+query_editor_activated_callback (NautilusQueryEditor *editor,
+				 NautilusWindowSlot *slot)
+{
+	if (slot->content_view != NULL) {
+		nautilus_view_activate_selection (slot->content_view);
+	}
+}
+
 static void
 query_editor_changed_callback (NautilusQueryEditor *editor,
 			       NautilusQuery *query,
@@ -205,6 +221,9 @@ nautilus_window_slot_set_query_editor_visible (NautilusWindowSlot *slot,
 		if (slot->qe_cancel_id == 0)
 			slot->qe_cancel_id = g_signal_connect (slot->query_editor, "cancel",
 							       G_CALLBACK (query_editor_cancel_callback), slot);
+		if (slot->qe_activated_id == 0)
+			slot->qe_activated_id = g_signal_connect (slot->query_editor, "activated",
+							       G_CALLBACK (query_editor_activated_callback), slot);
 
 	} else {
 		gtk_widget_hide (GTK_WIDGET (slot->query_editor));
@@ -212,6 +231,8 @@ nautilus_window_slot_set_query_editor_visible (NautilusWindowSlot *slot,
 		slot->qe_changed_id = 0;
 		g_signal_handler_disconnect (slot->query_editor, slot->qe_cancel_id);
 		slot->qe_cancel_id = 0;
+		g_signal_handler_disconnect (slot->query_editor, slot->qe_activated_id);
+		slot->qe_activated_id = 0;
 		nautilus_query_editor_set_query (slot->query_editor, NULL);
 	}
 }
