@@ -1454,15 +1454,16 @@ nautilus_window_key_press_event (GtkWidget *widget,
 		}
 	}
 
-	if (event->keyval != GDK_KEY_slash /* don't steal slash key event, used for "go to" */ &&
-	    event->keyval != GDK_KEY_Delete) {
-		if (nautilus_window_slot_handle_event (window->details->active_slot, event)) {
-			toggle_toolbar_search_button (window, TRUE);
-			return TRUE;
-		}
+	if (GTK_WIDGET_CLASS (nautilus_window_parent_class)->key_press_event (widget, event)) {
+		return TRUE;
 	}
 
-	return GTK_WIDGET_CLASS (nautilus_window_parent_class)->key_press_event (widget, event);
+	if (nautilus_window_slot_handle_event (window->details->active_slot, event)) {
+		toggle_toolbar_search_button (window, TRUE);
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 /*
@@ -1848,6 +1849,7 @@ nautilus_window_connect_content_view (NautilusWindow *window,
 				      NautilusView *view)
 {
 	NautilusWindowSlot *slot;
+	NautilusDirectory *directory;
 
 	g_assert (NAUTILUS_IS_WINDOW (window));
 	g_assert (NAUTILUS_IS_VIEW (view));
@@ -1869,9 +1871,14 @@ nautilus_window_connect_content_view (NautilusWindow *window,
        */
 	if (slot->pending_location == NULL) {
 		nautilus_window_load_view_as_menus (window);
-	}
+	} else {
+		directory = nautilus_directory_get (slot->pending_location);
+		if (!NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+			nautilus_view_grab_focus (view);
+		}
 
-	nautilus_view_grab_focus (view);
+		nautilus_directory_unref (directory);
+	}
 }
 
 void
